/// <reference types="node" />
import * as childProcess from 'child_process';
import TextStreamSearch from 'text-stream-search';
import { Env } from './env';
import { EndedNotification } from './ended-notification';
import { WriteStream } from './write-stream';
declare class ObservableProcess {
    cwd: string;
    ended: boolean;
    endedListeners: Array<(EndedNotification: any) => void>;
    env: Env;
    exitCode: number;
    killed: boolean;
    process: childProcess.ChildProcess;
    stdout: WriteStream;
    stderr: WriteStream;
    stdin: WriteStream;
    textStreamSearch: TextStreamSearch;
    verbose: boolean;
    constructor(args: {
        command?: string;
        commands?: string[];
        env?: Env;
        verbose?: boolean;
        cwd?: string;
        stdout?: WriteStream | null;
        stderr?: WriteStream | null;
    });
    private createStdOutErrStreamSearch;
    private forwardStreams;
    private determineRunnable;
    private spawnOptions;
    enter(text: string): void;
    fullOutput(): string;
    kill(): void;
    notifyEnded(): void;
    _onClose(exitCode: number): void;
    pid(): number;
    waitForEnd(): Promise<EndedNotification>;
    waitForText(text: string, timeout?: number): Promise<void>;
    resetOutputStreams(): void;
    _splitCommand(command: string | Array<string>): Array<string>;
}
export { Env, EndedNotification, WriteStream, ObservableProcess };
